@@ -7,7 +7,6 @@
  */
 
 #include <string.h>
-#include <math.h>
 #include "orcania.h"
 
 static const unsigned char base64_table[65] =
@@ -38,10 +37,10 @@ static int _o_base64_encode_agnostic(const unsigned char * src, size_t len, unsi
   *out_len = 0;
   while (end - in >= 3) {
     if (pos != NULL) {
-      *pos++ = table[in[0] >> 2];
-      *pos++ = table[((in[0] & 0x03) << 4) | (in[1] >> 4)];
-      *pos++ = table[((in[1] & 0x0f) << 2) | (in[2] >> 6)];
-      *pos++ = table[in[2] & 0x3f];
+      *pos++ = (unsigned char)(table[in[0] >> 2]);
+      *pos++ = (unsigned char)(table[((in[0] & 0x03) << 4) | (in[1] >> 4)]);
+      *pos++ = (unsigned char)(table[((in[1] & 0x0f) << 2) | (in[2] >> 6)]);
+      *pos++ = (unsigned char)(table[in[2] & 0x3f]);
     }
     (*out_len) += 4;
     in += 3;
@@ -53,12 +52,12 @@ static int _o_base64_encode_agnostic(const unsigned char * src, size_t len, unsi
 
   if (end - in) {
     if (pos != NULL) {
-      *pos++ = table[in[0] >> 2];
+      *pos++ = (unsigned char)(table[in[0] >> 2]);
     }
     (*out_len)++;
     if (end - in == 1) {
       if (pos != NULL) {
-        *pos++ = table[(in[0] & 0x03) << 4];
+        *pos++ = (unsigned char)(table[(in[0] & 0x03) << 4]);
         if (right_pad) {
           *pos++ = '=';
         }
@@ -70,8 +69,8 @@ static int _o_base64_encode_agnostic(const unsigned char * src, size_t len, unsi
       }
     } else {
       if (pos != NULL) {
-        *pos++ = table[((in[0] & 0x03) << 4) | (in[1] >> 4)];
-        *pos++ = table[(in[1] & 0x0f) << 2];
+        *pos++ = (unsigned char)(table[((in[0] & 0x03) << 4) | (in[1] >> 4)]);
+        *pos++ = (unsigned char)(table[(in[1] & 0x0f) << 2]);
       }
       (*out_len) += 2;
     }
@@ -141,9 +140,9 @@ static int o_base64_decode_agnostic(const unsigned char *src, size_t len, unsign
     }
     if (count == 4) {
       if (pos != NULL) {
-        *pos++ = (block[0] << 2) | (block[1] >> 4);
-        *pos++ = (block[1] << 4) | (block[2] >> 2);
-        *pos++ = (block[2] << 6) | block[3];
+        *pos++ = (unsigned char)((block[0] << 2)) | (unsigned char)((block[1] >> 4));
+        *pos++ = (unsigned char)((block[1] << 4)) | (unsigned char)((block[2] >> 2));
+        *pos++ = (unsigned char)((block[2] << 6)) | (unsigned char)(block[3]);
       }
       (*out_len)+=3;
       count = 0;
@@ -171,73 +170,22 @@ static int o_base64_decode_agnostic(const unsigned char *src, size_t len, unsign
   return 1;
 }
 
-/**
- * o_base64_encode - Base64 encode
- * @src: Data to be encoded
- * @len: Length of the data to be encoded
- * @out: Pointer to output variable
- * @out_len: Pointer to output length variable
- * Returns: 1 on success, 0 on failure
- *
- * The nul terminator is not included in out_len.
- */
 int o_base64_encode(const unsigned char * src, size_t len, unsigned char * out, size_t * out_len) {
   return _o_base64_encode_agnostic(src, len, out, out_len, base64_table, 1);
 }
 
-/**
- * o_base64_decode - Base64 decode
- * @src: Data to be decoded
- * @len: Length of the data to be decoded
- * @out: Pointer to output variable, you might add up to 2 bytes to out length than expected for safety
- * @out_len: Pointer to output length variable
- * Returns: 1 on success, 0 on failure
- *
- * The nul terminator is not included in out_len.
- */
 int o_base64_decode(const unsigned char *src, size_t len, unsigned char * out, size_t * out_len) {
   return o_base64_decode_agnostic(src, len, out, out_len, base64_table, 1);
 }
 
-/**
- * o_base64url_encode - Base64url encode (url format)
- * @src: Data to be encoded
- * @len: Length of the data to be encoded
- * @out: Pointer to output variable
- * @out_len: Pointer to output length variable
- * Returns: 1 on success, 0 on failure
- *
- * The nul terminator is not included in out_len.
- */
 int o_base64url_encode(const unsigned char * src, size_t len, unsigned char * out, size_t * out_len) {
   return _o_base64_encode_agnostic(src, len, out, out_len, base64_url_table, 0);
 }
 
-/**
- * o_base64url_decode - Base64 decode (url format)
- * @src: Data to be decoded
- * @len: Length of the data to be decoded
- * @out: Pointer to output variable, you might add up to 2 bytes to out length than expected for safety
- * @out_len: Pointer to output length variable
- * Returns: 1 on success, 0 on failure
- *
- * The nul terminator is not included in out_len.
- */
 int o_base64url_decode(const unsigned char *src, size_t len, unsigned char * out, size_t * out_len) {
   return o_base64_decode_agnostic(src, len, out, out_len, base64_url_table, 0);
 }
 
-/**
- * o_base64url_2_base64 - Convert a base64 url format to base64 format
- * @src: Data to be decoded
- * @len: Length of the data to be decoded
- * @out: Pointer to output variable
- * @out_len: Pointer to output length variable
- * Returns: 1 on success, 0 on failure
- *
- * The nul terminator is not included in out_len.
- * out must be at least len+2
- */
 int o_base64url_2_base64(const unsigned char *src, size_t len, unsigned char * out, size_t * out_len) {
   int res = 0;
   size_t index;
@@ -265,17 +213,6 @@ int o_base64url_2_base64(const unsigned char *src, size_t len, unsigned char * o
   return res;
 }
 
-/**
- * o_base64url_2_base64 - Convert a base64 format to base64 url format
- * @src: Data to be decoded
- * @len: Length of the data to be decoded
- * @out: Pointer to output variable
- * @out_len: Pointer to output length variable
- * Returns: 1 on success, 0 on failure
- *
- * The nul terminator is not included in out_len.
- * out must be at least len+2
- */
 int o_base64_2_base64url(const unsigned char *src, size_t len, unsigned char * out, size_t * out_len) {
   int res = 0;
   size_t index;
@@ -298,3 +235,107 @@ int o_base64_2_base64url(const unsigned char *src, size_t len, unsigned char * o
   
   return res;
 }
+
+int o_base64_encode_alloc(const unsigned char * src, size_t len, struct _o_datum * dat) {
+  int ret = 0;
+
+  if (dat != NULL) {
+    dat->size = 0;
+    if ((ret = o_base64_encode(src, len, NULL, &dat->size)) && dat->size) {
+      if ((dat->data = o_malloc(dat->size+2)) != NULL) {
+        memset(dat->data, 0, dat->size+2);
+        ret = o_base64_encode(src, len, dat->data, &dat->size);
+      } else {
+        ret = 0;
+      }
+    }
+  }
+  return ret;
+}
+
+int o_base64_decode_alloc(const unsigned char * src, size_t len, struct _o_datum * dat) {
+  int ret = 0;
+
+  if (dat != NULL) {
+    dat->size = 0;
+    if ((ret = o_base64_decode(src, len, NULL, &dat->size)) && dat->size) {
+      if ((dat->data = o_malloc(dat->size+2)) != NULL) {
+        memset(dat->data, 0, dat->size+2);
+        ret = o_base64_decode(src, len, dat->data, &dat->size);
+      } else {
+        ret = 0;
+      }
+    }
+  }
+  return ret;
+}
+
+int o_base64url_encode_alloc(const unsigned char * src, size_t len, struct _o_datum * dat) {
+  int ret = 0;
+
+  if (dat != NULL) {
+    dat->size = 0;
+    if ((ret = o_base64url_encode(src, len, NULL, &dat->size)) && dat->size) {
+      if ((dat->data = o_malloc(dat->size+2)) != NULL) {
+        memset(dat->data, 0, dat->size+2);
+        ret = o_base64url_encode(src, len, dat->data, &dat->size);
+      } else {
+        ret = 0;
+      }
+    }
+  }
+  return ret;
+}
+
+int o_base64url_decode_alloc(const unsigned char * src, size_t len, struct _o_datum * dat) {
+  int ret = 0;
+
+  if (dat != NULL) {
+    dat->size = 0;
+    if ((ret = o_base64url_decode(src, len, NULL, &dat->size)) && dat->size) {
+      if ((dat->data = o_malloc(dat->size+2)) != NULL) {
+        memset(dat->data, 0, dat->size+2);
+        ret = o_base64url_decode(src, len, dat->data, &dat->size);
+      } else {
+        ret = 0;
+      }
+    }
+  }
+  return ret;
+}
+
+int o_base64url_2_base64_alloc(const unsigned char * src, size_t len, struct _o_datum * dat) {
+  int ret = 0;
+  unsigned char * out = NULL;
+
+  if (dat != NULL) {
+    dat->size = 0;
+    if ((out = o_malloc(len+4)) != NULL) {
+      if ((ret = o_base64url_2_base64(src, len, out, &dat->size)) && dat->size) {
+        if ((dat->data = o_malloc(dat->size)) != NULL) {
+          memcpy(dat->data, out, dat->size);
+        }
+      }
+    }
+  }
+  o_free(out);
+  return ret;
+}
+
+int o_base64_2_base64url_alloc(const unsigned char * src, size_t len, struct _o_datum * dat) {
+  int ret = 0;
+  unsigned char * out = NULL;
+
+  if (dat != NULL) {
+    dat->size = 0;
+    if ((out = o_malloc(len)) != NULL) {
+      if ((ret = o_base64_2_base64url(src, len, out, &dat->size)) && dat->size) {
+        if ((dat->data = o_malloc(dat->size)) != NULL) {
+          memcpy(dat->data, out, dat->size);
+        }
+      }
+    }
+  }
+  o_free(out);
+  return ret;
+}
