@@ -19,13 +19,20 @@
  * 
  */
 
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <getopt.h>
 #include <ctype.h>
 #include <orcania.h>
 
+#ifdef _MSC_VER
+#include <io.h>
+#include <BaseTsd.h>
+typedef SSIZE_T ssize_t;
+#else
+#include <unistd.h>
+#endif
+
 #define _BASE64URL_VERSION "0.9"
 
 #define ACTION_ENCODE 0
@@ -33,6 +40,8 @@
 
 #define DEFAULT_WRAP 76
 
+#define SIZE 100
+
 static void print_output(const unsigned char * output, size_t output_len, unsigned long int wrap, short int ignore) {
   size_t i;
   for (i=0; i<output_len; i++) {
@@ -80,7 +89,7 @@ static unsigned char * get_file_content(const char * file_path, size_t * length)
   f = fopen (file_path, "rb");
   if (f) {
     fseek (f, 0, SEEK_END);
-    *length = ftell (f);
+    *length = (size_t)ftell (f);
     fseek (f, 0, SEEK_SET);
     buffer = o_malloc(((*length)+1)*sizeof(char));
     if (buffer) {
@@ -98,24 +107,27 @@ static unsigned char * get_file_content(const char * file_path, size_t * length)
 }
 
 static unsigned char * get_stdin_content(size_t * length) {
-  int size = 100;
-  unsigned char * out = NULL, buffer[size];
+  unsigned char * out = NULL, buffer[SIZE];
   ssize_t read_length;
 
   *length = 0;
-  while ((read_length = read(0, buffer, size)) > 0) {
-    out = o_realloc(out, (*length)+read_length+1);
-    memcpy(out+(*length), buffer, read_length);
-    (*length) += read_length;
+  while ((read_length = read(0, buffer, SIZE)) > 0) {
+    out = o_realloc(out, (*length)+(size_t)read_length+1);
+    memcpy(out+(*length), buffer, (size_t)read_length);
+    (*length) += (size_t)read_length;
     out[(*length)] = '\0';
   }
+  if ((*length) && (out[(*length)-1] == '\n' || out[(*length)-1] == '\r')) {
+    out[(*length)-1] = '\0';
+    (*length)--;
+  }
   return out;
 }
 
 int main(int argc, char ** argv) {
   short int action = ACTION_ENCODE, ignore = 0;
   unsigned long int wrap = DEFAULT_WRAP;
-  long int s_wrap = 0;
+  long unsigned int s_wrap = 0;
   const char * short_options = "d::i::w:f:v::h";
   int next_option, ret = 0, exit_loop = 0;
   char * file = NULL, * endptr = NULL, * tmp = NULL;
@@ -143,7 +155,7 @@ int main(int argc, char ** argv) {
         break;
       case 'w':
         s_wrap = strtoul(optarg, &endptr, 10);
-        if (*endptr == '\0' && s_wrap >= 0) {
+        if (*endptr == '\0' && s_wrap) {
           wrap = (unsigned long int)s_wrap;
         } else {
           print_help(stderr, argv[0]);
